-----------------------------------------------------------
----- ENTITY LAB1_TOP
-----------------------------------------------------------
LIBRARY ieee ;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use IEEE.MATH_REAL.ALL;

LIBRARY work ;
USE work.pk_bcd27seg.all;
USE work.PK_FREQ_DIV.all;
USE work.PK_COUNTER.all;

ENTITY LAB1_TOP IS
   	PORT(
			CLK50_IN   : IN STD_LOGIC;
         BUTTON1_IN : IN STD_LOGIC;
         BUTTON2_IN : IN STD_LOGIC;
         DISP_0     : OUT STD_LOGIC_VECTOR(6 DOWNTO 0);
         DISP_1     : OUT STD_LOGIC_VECTOR(6 DOWNTO 0)
   	);
END LAB1_TOP;

ARCHITECTURE struct of LAB1_TOP IS

-- Constants --------------------------------------
CONSTANT g_TB_FLAG: integer := 0; -- 0 VHDL para sintesis, 1 VHDL con modificaciones para simular


-- Signals ----------------------------------------
SIGNAL s_clk    : std_logic;

SIGNAL s_clr   : std_logic;
SIGNAL s_ena0   : std_logic;
SIGNAL s_ena1   : std_logic;

SIGNAL s_count0 : std_logic_vector(3 downto 0);
SIGNAL s_count1 : std_logic_vector(3 downto 0);

SIGNAL s_comp_out   : std_logic;

begin
	 s_clr <= not BUTTON2_IN;
     s_ena0 <= not BUTTON1_IN;
    	 
--- Reloj
	g_CLK_TB: if g_TB_FLAG = 1 generate	
		s_clk <= CLK50_IN;
	end generate g_CLK_TB;
	
	g_CLK_SYN: if g_TB_FLAG = 0 generate	
		FREQ_DIV_inst: FREQ_DIV port map(
			CLK_IN => CLK50_IN,
			CLK_OUT => s_clk
		);
	end generate g_CLK_SYN;

--- Contadores

    COUNTER_inst0 : COUNTER port map (
        CLEAR_IN   => s_clr,
        ENABLE_IN  => s_ena0,
        CLK_IN     => s_clk,
        COUNT_OUT  => s_count0
    );

    COUNTER_inst1 : COUNTER port map (
        CLEAR_IN   => s_clr,
        ENABLE_IN  => s_ena1,
        CLK_IN     => s_clk,
        COUNT_OUT  => s_count1
    );

    s_comp_out <= '1' when s_count0 = 9 else '0';
    s_ena1 <= s_comp_out and not BUTTON1_IN;
    
--- display
	g_DISP_TB: if g_TB_FLAG = 1 generate
		DISP_0 <= "000" & s_count0;
		DISP_1 <= "000" & s_count1;
	end generate g_DISP_TB;	 

	g_SYN_TB: if g_TB_FLAG = 0 generate
		bcd27seg_inst0: bcd27seg port map(
			bcd_in  => s_count0,
			seg_out => DISP_0
		);
		bcd27seg_inst1: bcd27seg port map(
			bcd_in  => s_count1,
			seg_out => DISP_1
		);
	end generate g_SYN_TB;	 




end struct;
